'use strict'

var Promise = require('bluebird')
var newify  = require('newify')
var Table   = require('alchemy/Table')
var Column  = require('alchemy/Column')

var addEntityMeta = require('./addEntityMeta')
var copy = require('./copy')

function unref(obj){
	return obj?
				JSON.parse(JSON.stringify(obj))
				:
				{}
}

function unique(arr){
    var result = []

    arr.forEach(function(value){
        if (result.indexOf(value) == -1){
            result.push(value)
        }
    })

    return result
}

module.exports = require('classy').define({
	alias: 'entity.Base',

	properties: {},

	autoCreate: false,

	statics: {

		init: function(){
			if (this.prototype.table){
				this.initTable()

				if (this.autoCreate){
					this.createTable()
				}
			}
		},

		fromState: function(state){
			return new this().setState(state)
		},

		initTable: function(){

			var proto = this.prototype
			var props = proto.properties

			var columns = Object.keys(props).map(function(name){

				var value = props[name]
				var args  = [value.mapping || name].concat(value)

				return newify(Column, args)
			})

			var args   = [proto.table, null].concat(columns)
			this.table = newify(Table, args)
		},

		toJSON: function(){
			var proto = this.prototype

		    return {
		    	properties: proto.properties,
		    	table     : proto.table,
		    	name      : proto.name
		    }
		},

		create: function(){
			var Create = this.table.create()
			var Insert = addEntityMeta(this)

			var Transaction = require('alchemy/Transaction')(Create, Insert)

			return Transaction.execute()
		}
	},

	init: function(values){
		// var result = {}

		this.initValues()
		this.setState()

		this.initProperties()
		this.set(values)
	},

	setState: function(state){
		state = unref(state)
		this._state = state

		this.setValues(state.values || {})

		return this
	},

	clone: function(){
	    return this.$ownClass.fromState(this.toState())
	},

	initValues: function(){
		Object.defineProperty(this, 'values', {
			get: function(values){
				return this._state.values
			},
			set: this.setValues.bind(this)
		})
	},

	setValues: function(values){
		var props = this.properties

		this._state.values = {}

		Object.keys(props).forEach(function(name){

			Object.defineProperty(this, name, {
				set: function(value){
					values[name] = value
				},
				get: function(){
					return values[name]
				}
			})

		}, this._state.values)
	},

	initTable: function(){
		var args = [ this.table, null ]
		var props = this._state.properties

		Object.keys(props).forEach(function(propName){
			var propColumn = props[propName]

			args.push(propColumn)
		})

		this.tableInstance = newify(Table, args)
	},

	initProperties: function(){

		this._state.properties = unref(this.properties)

		Object.keys(this._state.properties)
			.forEach(this.initProperty, this)

		// this.initTable()
	},

	initProperty: function(k) {
		var properties = this._state.properties

		properties[k] = this.getColumnForProperty(properties[k], k)

		var values = this.values

		if (values[k] === undefined){
			values[k] = undefined
		}
	},

	getColumnForProperty: function(propValue, propName){
		if (Array.isArray(propValue)){
			var arr = propValue.concat()
			var typeState = arr[0]
			var type      = require('./alchemy')().getType(typeState)

			arr[0]  = type.fromState(type)
			arr.unshift(propName)

			propValue = newify(Column, arr)
		}

		return propValue
	},

	setProperty: function(prop, value){
		this.values[prop] = value
	},

	set: function(values){
		if (arguments.length == 1 && typeof values == 'object'){
			Object.keys(values).forEach(function(k){
				this.set(k, values[k])
			}, this)

			return this
		}

		var k = arguments[0]
		var v = arguments[1]

		if (k in this._state.values){
			this._state.values[k] = v
		}

		return this
	},

	copy: function(obj){
	    copy(obj, this)

	    return this
	},

	getEmptyState: function(){
	    return {}
	},

	toState: function(){
	    return copy(this.cloneState(), this.getEmptyState())
	},

	cloneState: function(){
	    return this.toJSON()
	},

	toJSON: function(){
		var keys = Object.keys(this._state.properties)

		var result = {}

		keys.forEach(function(key){
	    	result[key] = this.values[key]
	    }, this)

	    return result
	}
})