var classy = require('classy')
var F      = require('functionally')
var copy   = require('./copy')
var CONFIG = require('./config')

var init         = require('./init')
var defineEntity = require('./defineEntity')

module.exports = function(engineConfig){
    if (typeof engineConfig == 'string'){
        engineConfig = {
            connectionString: engineConfig.connectionString
        }
    }

    engineConfig = engineConfig || {}

    function configure(config){
        copy(config, CONFIG)
    }

    function set(key, value){
        if (arguments.length == 1){
            Object.keys(key).forEach(function(k){
                set(k, key[k])
            }, this)

            return this
        }

        CONFIG[key] = value

        return this
    }

    function get(key){
        if (key && typeof key == 'object'){
            return Object.keys(key).map(get, this)
        }

        return CONFIG[key]
    }

    configure(engineConfig)

    var alchemy = require('./alchemy')()
    var EntityBase

    init()

    var logger = require('./loggerFactory')()

    global.log  = logger.info.bind(logger)
    global.warn = logger.warn

    var api = require('./api')

    return F.bindFunctionsOf({
        set: set,
        get: get,

        define: defineEntity,

        entity: {
            get User(){
                return require('./entities/User')
            }
        },

        api: {
            entity: {
                create   : api('entity/create'),
                list     : api('entity/list'),
                getByName: api('entity/getBy')('name'),
                getById  : api('entity/getById')
            },
            record: {
                create: api('record/create')
            }
        },


        Types : require('alchemy/Types'),
        Column: require('alchemy/Column'),
        Table : require('alchemy/Table')
    })
}