var CONFIG = require('../config')

var Promise = require('bluebird')
var Table   = require('alchemy/Table')
var Select  = require('alchemy/Select')
var Column  = require('alchemy/Column')
var Types   = require('alchemy/Types')

var Users = require('../entities/User')

module.exports = function(){

    var SelectExistsUsers = Select(
                                Column('table_name').as('name')
                            )
                            .from('information_schema.tables')
                            .where('table_schema')
                                .is('public')
                                .and('table_name')
                                .is('users')
                                .end()
                            .exists()

    return SelectExistsUsers
        .execute()
        .then(function(result){
            var exists = result.rows[0].exists

            exists?
                log('Users table exists.'):
                warn('Users table DOES NOT exist, will be created.')

            return new Promise(function(resolve, reject){

                var res = {
                    table: 'users',
                    exists: false
                }

                if (!exists){
                    return reject(res)
                }

                Select(Users.table.c.id.count('count'))
                    .execute()
                    .then(function(result){
                        var count = result.rows[0].count * 1

                        if (!count){
                            res.exists = true
                            res.count  = count
                            res.info = 'No user in table'
                            return reject(res)
                        }

                    }, reject)
            })


        }).catch(function(result){
            //the catch function is called when either the table does not exist
            //or if there is no users in the table
            //
            //If there is user, we just need to create an admin user.
            //If there is no users table, first we create the table and them create an admin user
            warn(result)

            if (!result.exists){
                log('Creating users table.')
            }

            var ensureTable = result.exists?
                                Promise.cast(true):
                                Users.create()

            return ensureTable
                    .then(function(){
                        return Users.insert({
                                    username: 'admin',
                                    password: CONFIG.adminPasswd
                                })
            })
        })
}